#ifndef INTERNAL_FCL_H
#define INTERNAL_FCL_H

#include "r_typedefs.h"

void Internal_FCL_OPEN(void);
void Internal_FCL_CLOSE(void);
void Internal_FCL_Init(void);
uint8_t Internal_FCL_Unlock(void);
uint8_t Internal_FCL_Erase(uint32_t blockID);
uint8_t Internal_FCL_Write(uint32_t address, uint8_t pbuf [], uint16_t length);
uint32_t Internal_FCL_Read32(uint32_t address);
void FCL_Ctrl1 (void);
void OptionByte_Handle(void);



#endif